'' =================================================================================================
''
''   File....... jm_hackable_ebadge_osh_demo__2015-09-18a.spin
''   Purpose.... Demo code for OSH Summit 2015
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 18 SEP 2015
''
'' =================================================================================================

{

  This badge demo has two modes:

  0 (default) This displays your name on the OLED. If you press the OSH logo on the badge, your
    contact information will be transmitted via IR to other badges (they should be within a few
    feet and facing you). Likewise, your badge can receive contact information from other badges.
    If your badge receives a contact that is already stored, it will be ignored. This allows the
    badge to store up to 100 unique contacts. These contacts are stored in the upper 32K of the
    64K EEPROM; this means they are safe from being overwritten should you wish to experiment with
    other applications on the badge. Restore this code to see the contacts. Viewing contacts is
    done by connecting the badge to a terminal set to 115200 baud. Open the terminal and then press
    any key to display your contacts.

  1 This mode displays X and Y accelerometer information as 0.0 (flat) to 90.0 (on an edge) degrees. 
    The value is signed based on the direction of tilt. The RGB LEDs (left for X, right for Y) will
    be green, yellow, or red depending on tilt (0 is green). The blue LEDs behave like a rotating
    bubble level.


    To switch between modes, shake the firmly for about 1/4 second. After switching modes, the
    badge will prevent mode swithcing for full second. Usually, a storng yet quick flick of the 
    wrist will change the badge mode (the RGB LED modules are the easiest way to detect the mode
    has changed).

}


con

  BADGE_DEMOS = 2

  #0, DEMO_IR, DEMO_ACC
  

con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us

  IR_BAUD  =   2_400  { max supported using IR connection }
  IR_FREQ  =  36_000  { matches receiver on DC22 badge }

  T_BAUD   = 115_200  { for terminal io } 
  

con { io pins }

  RX1      = 31                                                  ' programming / terminal
  TX1      = 30

  SDA      = 29                                                  ' eeprom / i2c
  SCL      = 28

  BTN_0    = 27                                                  ' touch buttons (right)
  BTN_1    = 26
  BTN_2    = 25

  IR_OUT   = 24                                                  ' IR coms
  IR_IN    = 23
  
  OLED_DAT = 22                                                  ' OLED connections 
  OLED_CLK = 21
  OLED_DC  = 20  
  OLED_RST = 19  
  OLED_CS  = 18

  BTN_5    = 17                                                  ' touch buttons (left)
  BTN_4    = 16
  BTN_3    = 15  

  TV_DAC2  = 14                                                  ' composite video (J503)  
  TV_DAC1  = 13
  TV_DAC0  = 12
  
  AUD_RT   = 10                                                  ' audio (J503)
  AUD_LF   =  9  

  BLU_CP2  =  8                                                  ' blue led charlieplex pins 
  BLU_CP1  =  7 
  BLU_CP0  =  6

  BTN_OS   =  5                                                  ' open source logo button
  
  ACC_INT  =  4                                                  ' accelerometer interrupt in
  
  RGB_CP2  =  3                                                  ' rgb led charlieplex pins 
  RGB_CP1  =  2 
  RGB_CP0  =  1

  BATT_MON =  0                                                  ' battery monitor(go/no-go)


con { buttons masks }
  
  PB0_MASK  = %0_000_001                                         ' upper right
  PB1_MASK  = %0_000_010                                         ' middle right
  PB2_MASK  = %0_000_100                                         ' lower right
  PB3_MASK  = %0_001_000                                         ' lower left
  PB4_MASK  = %0_010_000                                         ' middle left
  PB5_MASK  = %0_100_000                                         ' upper left
  PB6_MASK  = %1_000_000                                         ' open source logo

  OFF_MASK  = PB5_MASK | PB0_MASK
  

dat { touch pads configuration }

  TPCount       byte    7
  TPPins        byte    BTN_OS, BTN_5, BTN_4, BTN_3, BTN_2, BTN_1, BTN_0
  TPDischarge   byte    15
  

con { led control }

  #0, BLUE_0, BLUE_1, BLUE_2, BLUE_3, BLUE_4, BLUE_5             ' charlieplex control for blue leds
  #0, RGB_B1, RGB_G1, RGB_R1, RGB_B2, RGB_G2, RGB_R2             ' charlieplex control for rgb leds


con

  #0, NO, YES

  SHAKE_DELAY   = 25                                             ' read for shake every 25ms
  SHAKE_EVENT   = 150                                            ' switch mode if shaking for ~150ms
  SHAKE_HOLDOFF = 1000                                           ' ignore shake detection for 1s after app switch

  #-1, MSG_LOGO                                                  ' display messages
   #0, MSG_NAME
   #1, MSG_TX_NOW, MSG_TX_DONE
   #3, MSG_RX_NOW, MSG_RX_DONE, MSG_RX_DUP, MSG_RX_ERROR
   #9, MSG_WIPE

  NAME_CHARS  = 8                                                ' using 2x8 Parallax font mode in OLED
  SCROLL_MS   = 500
  
  CONTACT_LEN  = ee#PG_SIZE                                      ' use full page for access efficiency
  MAX_CONTACTS = 100                                             ' max size of contacts db
  LAST_CONTACT = MAX_CONTACTS - 1

  STX = 2                                                        ' start of text
  ETX = 3                                                        ' end of text
  EOT = 4                                                        ' end of transmission

  #0, RX_NEW, RX_DUPLICATE, RX_ERROR                             ' contact rx states
  
  TX_HOLDOFF = 1                                                 ' wait 1s before next TX attempt

  
obj

' main                                                           ' * master Spin cog
  time   : "jm_time"                                             '   timing and delays
  dtmr   : "jm_time"                                             '   display state timer
  scrtmr : "jm_time"                                             '   scroll timer
  shktmr : "jm_time"                                             '   shake hold-off timer
  io     : "jm_io"                                               '   io control
  ee     : "jm_24xx512"                                          '   64k eeprom access
  acc    : "jm_mma7660fc"                                        '   MMA7660 accelerometer
  pads   : "jm_touchpads"                                        '   touchpad input
  oled   : "oled_asmfast"                                        ' * OLED driver (TEMPORARY)
  leds   : "jm_ebadge_leds"                                      ' * charlieplexing driver for LEDs
  ir     : "jm_ir_hdserial"                                      ' * half-duplex serial via IR
  term   : "jm_fullduplexserial"                                 ' * full-duplex serial via terminal
                                                                  
' * Uses cog when loaded                                          


var

  long  cpcog                                                    ' charlieplex cog (>0 if loaded)

  long  badgemode                                                ' 0 for name/ir, 1 for accelerometer

  long  dstate                                                   ' display state
  long  msgms                                                    ' message/display milliseconds

  long  numcontacts                                              ' number of contacts in EEPROM

  long  scrollidx                                                ' scroll position
  long  scrollidxmax
  byte  scroll[32]                                               ' scroll buffer

  byte  buf1[CONTACT_LEN]                                        ' rx buffer
  byte  buf2[CONTACT_LEN]                                        ' check buffer


con

  SCROLL_NAME = NO                                               ' scroll status for message mode 0

  
dat { name & messages }

' Specify scrolling lines without any space padding
' -- maximum length of either name is 24
' -- SCROLL_NAME = YES

  FName         byte    "JonnyMac", 0  
  LName         byte    "King Of Propellers", 0


' Non-scrolling messages
' -- Lines should be limited to 8 characters
'    * will be truncated if longer 
 
  Name_0        byte    "Jon     ", 0                            ' SCROLL_NAME = NO 
  Name_1        byte    "McPhalen", 0  

  Msg1_0        byte    "Sending", 0
  Msg1_1        byte    "Contact", 0
  
  Msg2_0        byte    "Contact", 0
  Msg2_1        byte    "Sent", 0

  Msg3_0        byte    "Getting", 0
  Msg3_1        byte    "Contact", 0
  
  Msg4_0        byte    "Contact", 0
  Msg4_1        byte    "Received", 0
  
  Msg5_0        byte    "Already", 0
  Msg5_1        byte    "Stored!", 0

  Msg6_0        byte    "Receive", 0
  Msg6_1        byte    "Error", 0

  Msg9_0        byte    "Database", 0
  Msg9_1        byte    "  Wipe", 0


dat { contact information }

' Suggested structure of info in name, email, web site, telephone
' -- separate elements with 13 (carriage return)
' -- max size is CONTACT_LEN-1 (zero needed for termination)

  MyInfo        byte    "JonnyMac", 13 
                byte    "jon.mcphalen@gmail.com", 13 
                byte    "www.jonmcphalen.com", 13 
                byte    "310-555-1212", 0
   
 
' Note: If ResetDB is set to YES (any non-zero value), the contacts database in upper EEPROM will be
'       wiped clean
  
  ResetDB       byte    YES


pub main | t

  setup                                                          ' setup io and badge objects

  check_db                                                       ' check for database reset  

  dstate := MSG_LOGO                                             ' show OSH logo 1st
  update_display(1000)

  lamp_test                                                      ' test leds

  repeat
    check_shake                                                  ' want to change mode?

    case badgemode
      DEMO_IR  : ir_demo
      DEMO_ACC : acc_demo


pub check_shake | total, t 

'' Check shake for application switch

  if (shktmr.millis < 0)                                         ' shake hold-off still running?
    return

  total := 0                                                     ' clear shake count

  t := cnt                                                       ' sync loop timing
  repeat
    ifnot (acc.shake(-1))                                        ' read sensor for shake
      quit                                                       '  if no, abort

    total += SHAKE_DELAY                                         ' update shake timing
    if (total => SHAKE_EVENT)                                    ' if threshold reached
      if (++badgemode == BADGE_DEMOS)                            ' switch to next app
        badgemode := 0                                           '  rollover if needed
      shktmr.set(-SHAKE_HOLDOFF)                                 ' set shake hold-off
      if (badgemode == DEMO_IR)                                  ' back to name display?
        leds.set_all(0)                                          ' acc leds off
        if (SCROLL_NAME == NO)                                   ' not scrolling name?
          dstate := !MSG_NAME                                    '  force display refresh
          msgms := negx
      quit
      
    waitcnt(t += (MS_001 * SHAKE_DELAY))                         ' run loop every SHAKE_DELAY ms 


pub ir_demo

'' Display badge owner name and exchange contact info via IR
                                                                  
  if ((dstate <> MSG_NAME) and (dtmr.millis => msgms))           ' time to clear message
    dstate := MSG_NAME                                           ' go back to name
    update_display(posx)                                          
    leds.set_all(0)                                               
    scrollidx := -NAME_CHARS                                     ' reset for scrolling
    scrtmr.set(-SCROLL_MS)                                        
  else                                                            
    check_scroll

  check_receive                                                  ' data coming in?                   
  check_transmit                                                 ' time to send my contact info
  display_contacts                                               ' show contacts in terminal                                                     


pub check_scroll | len

'' Scroll FName and LName through display
'' -- scroll only in name display mode
'' -- scrolling must be enabled

  if ((dstate == MSG_NAME) and (SCROLL_NAME == YES))             ' scrolling enabled?        
    if (scrtmr.millis => 0)                                      ' if delay done             
      if (++scrollidx > scrollidxmax)                            ' at end of scroll window?                                                      
        scrollidx := -NAME_CHARS                                 '  reset (for scroll on)
                                                                     
      bytefill(@scroll, " ", 32)                                 ' fill scroll buf with spaces
      len := strsize(@FName)                                     ' get length of segment     
      if (scrollidx < 0)                                         ' scroll on?
        bytemove(@scroll + ||scrollidx, @FName, len)             ' put text on right side of display
      else                                                       ' scroll off
        if (scrollidx < len)                                     ' anything viewable for this index?
          bytemove(@scroll, @FName + scrollidx, len-scrollidx)   ' display current segment                                     
      oled.write2x8String(@scroll, 8, 0)                         ' write in oled

      bytefill(@scroll, " ", 32)
      len := strsize(@LName)                                  
      if (scrollidx < 0)                                         
        bytemove(@scroll + ||scrollidx, @LName, len)
      else
        if (scrollidx < len)                                     
          bytemove(@scroll, @LName + scrollidx, len-scrollidx) 
      oled.write2x8String(@scroll, 8, 1)                         

      oled.updateDisplay                                         ' refresh display
      scrtmr.set(-SCROLL_MS)                                     ' reset scrolling timer
       

pub display_contacts  | k, idx

'' Check for input from terminal
'' -- if key is pressed, display current contacts
 
  k := term.rxcheck                                              ' display contacts?
  if (k < 0)
    return

  term.tx(term#CLS)                                              ' clear PST
  
  term.str(string("ME"))                                         ' show my information
  term.tx(term#CR)
  repeat 40
    term.tx("-")
  term.tx(term#CR)
  term.str(@MyInfo)

  repeat 4
    term.tx(term#CR)

  term.str(string("CONTACTS: " ))                               ' show # contacts
  term.dec(numcontacts)
  term.tx(term#CR)
  repeat 40
    term.tx("-")
  term.tx(term#CR)

  idx := 0                                                       ' set index to start of contacts
  
  repeat numcontacts                                             ' loop through contacts
    read_record(idx, @buf1)                                      ' copy from EE to RAM
    if (strsize(@buf1) =< 0)
      term.str(string("Contact Error ("))
      term.dec(idx)
      term.tx(")")
    else
      term.str(@buf1)                                            ' display on terminal
    repeat 2                                                     ' pad with blank lines
      term.tx(term#CR)
    ++idx


pub check_receive | c, idx, check, checksum, found

'' Looks for input from IR uart
'' -- contact string is framed with STX and ETX bytes

  if (numcontacts == MAX_CONTACTS)                               ' if no room in db
    return                                                       '  abort
 
  c := ir.rxcheck                                                ' any ir traffic?
  if (c <> STX)                                                  ' if not STX, abort
    return

  dstate := MSG_RX_NOW
  update_display(posx)  
  leds.set_rgbx(leds#GREEN, leds#GREEN)                          ' show rx'ing

  ' rx bytes into buffer until t/o or ETX

  bytefill(@buf1, 0, CONTACT_LEN)                                ' clear rx buffer
  idx := 0                                                       ' set index to beginning
  check := 0                                                     ' clear running checksum
  
  repeat
    c := ir.rxtime(10)                                           ' use time-out in case packet breaks
    if (c < 0)                                                   ' time out
      rx_status(RX_ERROR)                                        ' show error
      return
    elseif (c == ETX)                                            ' end of contact transmission
      quit
    else
      buf1[idx] := c                                             ' add byte to buffer
      check += c                                                 ' update working checksum
      if (++idx == CONTACT_LEN-1)                                ' max length reached?
        quit                                                     '  yes, done

  check := $FF - (check & $FF)                                   ' clean-up working checksum
    
  checksum := ir.rxtime(10)                                      ' get checksum from sender
  if ((checksum < 0) or (checksum <> check))                     ' validate (no timeout or mismatch)
    rx_status(RX_ERROR) 
    return

  c := ir.rxtime(10)                                             ' get EOT
  if (c <> EOT)                                                  ' validate       
    rx_status(RX_ERROR) 
    return                                                   

  ' make sure we have something in the buffer

  if (strsize(@buf1) == 0)
    rx_status(RX_ERROR) 
    return  
    
  ' buffer now holds new contact

  idx := 0                                                       ' start at 1st contact
  found := false                                                 ' assume not found

  repeat numcontacts                                             ' loop through contacts
    read_record(idx, @buf2)                                      ' read contact from database
    if (strcomp(@buf1, @buf2))                                   ' if match in contacts db       
      found := true                                              '  mark as found                
      quit
    else
      ++idx

  ifnot (found)
    write_record(numcontacts, @buf1)                             ' write new contact to EE    
    ee.wr_long($8000, ++numcontacts)                             ' update count and save
    rx_status(RX_NEW)
  else      
    rx_status(RX_DUPLICATE)

  leds.set_all(0)        

 
pub rx_status(status) | color

'' Report RX status on OLED and via flashing RGB

  if (status == RX_NEW)                                          ' new contact rx'd
    dstate := MSG_RX_DONE
  elseif (status == RX_DUPLICATE)                                ' repeat contact
    dstate := MSG_RX_DUP
  elseif (status == RX_ERROR)                                    ' rx problem
    dstate := MSG_RX_ERROR
  else
    leds.set_all(0)                                              ' clear everything
    dstate := MSG_LOGO                                           ' force refresh in main loop
    msgms := 0
    return

  update_display(1000)                                           ' show status message

  color := lookupz(status : leds#GREEN, leds#YELLOW, leds#RED)

  flash_rgbs(color) 
       

pub check_transmit | idx, c, checksum

'' Send MyInfo string between STX and ETX framing bytes
'' -- keep running checksum
'' -- tx checkum, then EOT to end transmission

  if (time.millis < 0)                                           ' in tx hold-off period?
    return

  repeat 3
    if (pads.read_pads <> PB6_MASK)                              ' OSH logo pressed?
      return
    time.pause(15)
         
  dstate := MSG_TX_NOW                                           ' show sending
  update_display(posx)

  leds.set_rgbx(leds#GREEN, leds#GREEN) 

  ' send contact info between framing bytes

  ir.tx(STX)                                                     ' start of text
  idx := 0                                                       ' 
  checksum := 0                                                  ' clear checksum
  
  repeat
    c := MyInfo[idx]                                             ' get a character
    if (c == 0)                                                  ' if end of info
      quit                                                       '  exit
    else
      ir.tx(c)                                                   ' tx the character
      checksum += c                                              ' add to checksum
      if (++idx == CONTACT_LEN)                                  ' limit transmission length
        quit
  
  ir.tx(ETX)                                                     ' end of text
  checksum := $FF - (checksum & $FF)                             ' create checksum
  ir.tx(checksum)                                                ' send it
  ir.tx(EOT)                                                     ' end of transmission                                                    
  ir.txflush                                                     ' let it finish

  dstate := MSG_TX_DONE                                          ' show sending
  update_display(1000)
  flash_rgbs(leds#GREEN)    

  time.set_secs(-TX_HOLDOFF)                                     ' disable tx


pub flash_rgbs(color)

  repeat 6
    leds.set_rgbx(color, color)          
    time.pause(75)                                 
    leds.set_all(0)                                
    time.pause(75)                                 


pub update_display(ms) | p_str0, p_str1

'' Displays "screen" based on dstate
'' -- if no match, Propeller beanie logo displayed
'' -- ms is milliseconds to display message   

  case dstate
    MSG_NAME:
      if (SCROLL_NAME == NO)                                     ' non-scrolling name
        p_str0 := @Name_0
        p_str1 := @Name_1 
      else
        bytefill(@scroll, " ", 8)                                ' blanks
        p_str0 := @scroll
        p_str1 := @scroll
        
    MSG_TX_NOW:
      p_str0 := @Msg1_0                                          ' sending contact
      p_str1 := @Msg1_1

    MSG_TX_DONE:
      p_str0 := @Msg2_0                                          ' contact sent
      p_str1 := @Msg2_1
      
    MSG_RX_NOW:
      p_str0 := @Msg3_0                                          ' receiving contact
      p_str1 := @Msg3_1

    MSG_RX_DONE:
      p_str0 := @Msg4_0                                          ' contact received
      p_str1 := @Msg4_1
      
    MSG_RX_DUP:
      p_str0 := @Msg5_0                                          ' already stored
      p_str1 := @Msg5_1

    MSG_RX_ERROR:
      p_str0 := @Msg6_0                                          ' receive error
      p_str1 := @Msg6_1
      
    MSG_WIPE:
      p_str0 := @Msg9_0                                          ' database wipe
      p_str1 := @Msg9_1

    other: { graphic }
      bytemove(oled.getBuffer, @Logo, 1024)
    
  if (dstate => 0)
    'oled.clearDisplay
    bytefill(oled.getBuffer, 0, 1024)
    oled.write2x8String(p_str0, strsize(p_str0) <# 8, 0)  
    oled.write2x8String(p_str1, strsize(p_str1) <# 8, 1)

  oled.updateDisplay

  msgms := ms                                                    ' set message time
  dtmr.start                                                     ' restart display timer


pub read_record(idx, p_dest) | addr

'' Read record idx to destination
'' -- p_dest is pointer to destination

  bytefill(p_dest, 0, CONTACT_LEN)                               ' clear destination

  if ((idx < 0) or (idx > LAST_CONTACT))                         ' validate record idex
    return

  addr := $8000 + ((idx+1) * CONTACT_LEN)                        ' calculate EE address

  ee.rd_block(addr, CONTACT_LEN, p_dest)                         ' read from EE to destination  


pub write_record(idx, p_src) | addr

'' Writes record idx to EEPROM
'' -- idx is 0 to LAST_CONTACT 
'' -- p_src is pointer to data to write

  if ((idx < 0) or (idx > LAST_CONTACT))                         ' validate record idex
    return

  addr := $8000 + ((idx+1) * CONTACT_LEN)                        ' calculate EE address

  ee.wr_block(addr, CONTACT_LEN, p_src)                          ' write source to EE    
  

pub check_db | check

'' Checks to see if contacts database should be wiped clean

  numcontacts := ee.rd_word($8000)                               ' should be 0 to MAX_CONTACTS
  if (numcontacts > MAX_CONTACTS)
    wipe_db                       
    return

  check := ee.rd_word($8002)                                     ' should be 0 (new ee = $FFFF)
  if (check <> $0000)
    wipe_db
    return
  
  if (ResetDB <> NO) 
    wipe_db 


pub wipe_db | idx, addr, cycle

'' Clears contacts database and number of contacts

  dstate := MSG_WIPE                                             ' alert user we're clearing db
  update_display(posx)

  bytefill(@buf1, 0, CONTACT_LEN)                                ' clear buffer

  cycle := 0
  
  repeat idx from 0 to MAX_CONTACTS                              ' contacts + 1
    leds.set_blue(1 << cycle)                                    ' animate blue LEDs
    if (++cycle == 6)
      cycle := 0
    addr := $8000 + (idx * CONTACT_LEN)                          ' calculate ee address
    ee.wr_block(addr, CONTACT_LEN, @buf1)                        ' write block of 0s to EE  

  ResetDB := NO                                                  ' no need to wipe again
  ee.wr_byte(@ResetDB, ResetDB)                                  ' mark for next reset
  
  numcontacts := 0


dat { accelerometer angle table }

  Angle_X       byte    "X   0.0", $B0, 0                        ' $B0 is degrees symbol in Parallax font
  Angle_Y       byte    "Y   0.0", $B0, 0

  ' adapted from Appendix C of MMA7660FC documentation
  ' -- rounded to 0.1 degrees

  AngleTable    word      0,  27,  54,  81, 108, 136, 163  
                word    192, 220, 250, 280, 310, 342, 375
                word    410, 447, 486, 528, 575, 630, 696 
                word    799, 900
  

pub acc_demo | sensor, x, y

'' Display X & Y accelerometer axes in %

  acc.read_all_raw(@sensor)                                      ' read raw sensor bytes

  x := sensor << 26 ~> 26                                        ' extract signed X axis (sensor.byte[0])
  y := sensor << 18 ~> 26                                        ' extract signed Y axis (sensor.byte[1]) 
  
  update_acc_leds(x, y)   

  x := x * AngleTable[||x <# 22] / ||x                           ' convert to signed angle
  value_nn_n(x, @Angle_X[2])                                     ' update display string
  
  y := y * AngleTable[||y <# 22] / ||y
  value_nn_n(y, @Angle_Y[2]) 

  oled.write2x8String(@Angle_X, 8, 0)                            ' write new values
  oled.write2x8String(@Angle_Y, 8, 1)                                 
  oled.updateDisplay                                             ' refresh oled 
  

pub value_nn_n(value, p_str) | zflag

'' Write 3-digit justified value into p_str
'' -- format it -NN.N

  zflag := false

  if (value < 0)
    byte[p_str++] := "-"
    -value
  else
    byte[p_str++] := " "

  if (value => 100)
    byte[p_str++] := "0" + (value / 100)                         ' 10s digit (or space if < 100)
  else
    byte[p_str++] := " "

  byte[p_str++] := "0" + ((value // 100) / 10)                   ' 1s digit

  byte[p_str++] := "."                                           ' decimal separator  

  byte[p_str] := "0" + (value // 10)                             ' 0.1s digit


pub update_acc_leds(x, y)

'' Use RGB and blue leds to assist in level display

  case x
    -2..2 : leds.set_rgb1(leds#GREEN)                            ' level
    -6..6 : leds.set_rgb1(leds#YELLOW)                           ' a little off
    other : leds.set_rgb1(leds#RED)                              ' way off

  case y
    -2..2 : leds.set_rgb2(leds#GREEN)
    -6..6 : leds.set_rgb2(leds#YELLOW)
    other : leds.set_rgb2(leds#RED)


  if (y < -2)
    if (x < -2)
      leds.set_blue(1 << BLUE_5)
      
    elseif (x > 2)
      leds.set_blue(1 << BLUE_3)
       
    else
      leds.set_blue(1 << BLUE_4)  
    
  elseif (y > 2)
    if (x < -2)
      leds.set_blue(1 << BLUE_0)
      
    elseif (x > 2)
      leds.set_blue(1 << BLUE_2)
       
    else
      leds.set_blue(1 << BLUE_1)  

  else
    if (x < -2)
      leds.set_blue(1 << BLUE_5 | 1 << BLUE_0)
                                        
    elseif (x > 2)                   
      leds.set_blue(1 << BLUE_3 | 1 << BLUE_2)
                                        
    else                                
      leds.set_blue(1 << BLUE_4 | 1 << BLUE_1)  



pub lamp_test

'' Quick test of LEDs

  leds.set_blue(%111111)  

  leds.set_rgbx(leds#RED, leds#RED)
  time.pause(500)
  
  leds.set_rgbx(leds#GREEN, leds#GREEN)
  time.pause(500)
    
  leds.set_rgbx(leds#BLUE, leds#BLUE)
  time.pause(500)
  
  leds.set_all(0)


pub setup                              

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed
  dtmr.start
  scrtmr.start
  shktmr.start

  io.start($0000, $0000)                                         ' clear all pins (this cog)
  
  ee.start(%000)                                                 ' connect to boot eeprom

  acc.start(SCL, SDA)                                            ' connect to accelerometer

  pads.start(TPCount, @TPPins, TPDischarge)                      ' setup touchpad interface

  oled.init(OLED_CS, OLED_DC, OLED_DAT, OLED_CLK, OLED_RST, {    ' start buffered oled driver
}           oled#SSD1306_SWITCHCAPVCC, oled#TYPE_128X64)
  oled.AutoUpdateOff  

  cpcog := leds.start                                            ' start charlieplexing driver
  
  ir.start(IR_IN, IR_OUT, IR_BAUD, IR_FREQ)                      ' start IR serial  
  
  term.start(RX1, TX1, %0000, T_BAUD)                            ' start serial for terminal

  if (strsize(@FName) > 24)
    byte[@FName][24] := 0                                        ' truncate if needed

  if (strsize(@LName) > 24) 
    byte[@LName][24] := 0     

  scrollidxmax :=  strsize(@FName)                               ' find max idx for scrolling
  scrollidxmax #>= strsize(@LName)


con

  { --------------------------- }
  {  E B A D G E   B A S I C S  }
  { --------------------------- }


pub set_1_blue(idx)                                              ' LOCKED 30 JUN 2015  

'' Set one of the six blue LEDs
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?
    if ((idx => BLUE_0) and (idx =< BLUE_5))                     ' valid element?
      leds.set_blue(1 << idx)                                    ' set via mask
    else
      leds.set_blue(%000000)                                     ' clear all 
    return 

  ' if no driver loaded, use direct control

  case idx
    BLUE_0:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_1:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L    
      dira[BLU_CP2..BLU_CP0] := %101

    BLUE_2:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H 
      dira[BLU_CP2..BLU_CP0] := %101      

    BLUE_3:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z  
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_4:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[BLU_CP2..BLU_CP0] := %011
      
    BLUE_5:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H
      dira[BLU_CP2..BLU_CP0] := %011  

    other:
      outa[BLU_CP2..BLU_CP0] := %000                             ' disable all
      dira[BLU_CP2..BLU_CP0] := %000         


pub set_1_rgb(idx)                                               ' LOCKED 30 JUN 2015  

'' Set one of the LEDs within the RGB modules
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs/colors use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?      
    if ((idx => RGB_B1) and (idx =< RGB_R2))                     ' valid element?                  
      leds.set_rgb(1 << idx)                                     ' set via mask                    
    else                                                                                           
      leds.set_rgb(%000000)                                      ' clear all                       
    return

  ' if no driver loaded, use direct control

  case idx
    RGB_B1:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %011
  
    RGB_R1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[RGB_CP2..RGB_CP0] := %110

    RGB_B2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H  
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H 
      dira[RGB_CP2..RGB_CP0] := %011      

    RGB_R2:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z
      dira[RGB_CP2..RGB_CP0] := %110  
  
    other:
      outa[RGB_CP2..RGB_CP0] := %000                             ' disable all
      dira[RGB_CP2..RGB_CP0] := %000 


dat { OSH Logo }

  Logo          byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $F0, $FC, $FC, $FC, $FC
                byte    $FC, $FC, $FC, $FC, $FC, $F0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $C0, $E0, $F0, $F8, $FC, $FE, $FE, $FC, $FC, $F8
                byte    $F0, $E0, $E0, $C0, $C0, $E0, $E0, $E0, $F0, $F0, $FC, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $F0, $E0, $E0, $E0, $C0, $C0, $E0, $E0
                byte    $F0, $F8, $FC, $FC, $FE, $FE, $FC, $F8, $F0, $E0, $C0, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $01, $03, $0F, $1F, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $1F, $0F, $03, $01, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80
                byte    $80, $80, $80, $C0, $C0, $C0, $C0, $C0, $F0, $FC, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $3F, $1F, $0F, $07, $07, $07, $03
                byte    $03, $03, $07, $07, $07, $0F, $1F, $3F, $7F, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $C0, $C0, $C0, $C0, $C0, $80, $80
                byte    $80, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $81, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $81, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01
                byte    $01, $01, $01, $01, $01, $03, $03, $03, $0F, $7F, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FE, $FC, $F8, $F0, $20, $00, $00, $00
                byte    $00, $00, $00, $00, $20, $F0, $F8, $FC, $FE, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $0F, $03, $03, $03, $01, $01, $01, $01
                byte    $01, $01, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $80, $E0, $F0, $FC, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $1F, $07, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $07, $1F, $7F, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $E0, $80, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $01, $03, $07, $0F, $1F, $3F, $7F, $3F, $1F, $0F
                byte    $0F, $07, $03, $03, $07, $07, $0F, $03, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $03, $0F, $07, $07, $03, $03, $07
                byte    $0F, $0F, $1F, $3F, $7F, $3F, $1F, $0F, $07, $03, $01, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}