con

  BADGE_DEMOS = 2  

  #0, DEMO_IR, DEMO_ACC
  

con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us

  IR_BAUD  =   2_400  { max supported using IR connection }
  IR_FREQ  =  38_000  { matches receiver on DC22 badge }

  T_BAUD   = 115_200  { for terminal io } 
  

con { io pins }

  RX1      = 31                                                  ' programming / terminal
  TX1      = 30

  SDA      = 29                                                  ' eeprom / i2c
  SCL      = 28

  BTN_0    = 27                                                  ' touch buttons (right)
  BTN_1    = 26
  BTN_2    = 25

  IR_OUT   = 24                                                  ' IR coms
  IR_IN    = 23
  
  OLED_DAT = 22                                                  ' OLED connections 
  OLED_CLK = 21
  OLED_DC  = 20  
  OLED_RST = 19  
  OLED_CS  = 18

  BTN_5    = 17                                                  ' touch buttons (left)
  BTN_4    = 16
  BTN_3    = 15  

  TV_DAC2  = 14                                                  ' composite video (J503)  
  TV_DAC1  = 13
  TV_DAC0  = 12
  
  AUD_RT   = 10                                                  ' audio (J503)
  AUD_LF   =  9  

  BLU_CP2  =  8                                                  ' blue led charlieplex pins 
  BLU_CP1  =  7 
  BLU_CP0  =  6

  BTN_OS   =  5                                                  ' open source logo button
  
  ACC_INT  =  4                                                  ' accelerometer interrupt in
  
  RGB_CP2  =  3                                                  ' rgb led charlieplex pins 
  RGB_CP1  =  2 
  RGB_CP0  =  1

  BATT_MON =  0                                                  ' battery monitor(go/no-go)


con { buttons masks }
  
  PB0_MASK  = %0_000_001                                         ' upper right
  PB1_MASK  = %0_000_010                                         ' middle right
  PB2_MASK  = %0_000_100                                         ' lower right
  PB3_MASK  = %0_001_000                                         ' lower left
  PB4_MASK  = %0_010_000                                         ' middle left
  PB5_MASK  = %0_100_000                                         ' upper left
  PB6_MASK  = %1_000_000                                         ' open source logo

  OFF_MASK  = PB5_MASK | PB0_MASK
  

dat { touch pads configuration }

  TPCount       byte    7
  TPPins        byte    BTN_OS, BTN_5, BTN_4, BTN_3, BTN_2, BTN_1, BTN_0
  TPDischarge   byte    15
  

con { led control }

  #0, BLUE_0, BLUE_1, BLUE_2, BLUE_3, BLUE_4, BLUE_5             ' charlieplex control for blue leds
  #0, RGB_B1, RGB_G1, RGB_R1, RGB_B2, RGB_G2, RGB_R2             ' charlieplex control for rgb leds


con

  #0, NO, YES

  SHAKE_DELAY   = 25                                             ' read for shake every 25ms
  SHAKE_EVENT   = 150                                            ' switch mode if shaking for ~150ms
  SHAKE_HOLDOFF = 1000                                           ' ignore shake detection for 1s after app switch

  #-1, MSG_LOGO                                                  ' display messages
   #0, MSG_NAME
   #1, MSG_TX_NOW, MSG_TX_DONE
   #3, MSG_RX_NOW, MSG_RX_DONE, MSG_RX_DUP, MSG_RX_ERROR
   #9, MSG_WIPE

  NAME_CHARS  = 8                                                ' using 2x8 Parallax font mode in OLED
  SCROLL_MS   = 500
  
  CONTACT_LEN  = ee#PG_SIZE                                      ' use full page for access efficiency
  MAX_CONTACTS = 100                                             ' max size of contacts db
  LAST_CONTACT = MAX_CONTACTS - 1

  STX = 2                                                        ' start of text
  ETX = 3                                                        ' end of text
  EOT = 4                                                        ' end of transmission

  #0, RX_NEW, RX_DUPLICATE, RX_ERROR                             ' contact rx states
  
  TX_HOLDOFF = 1                                                 ' wait 1s before next TX attempt

  
obj

' main                                                           ' * master Spin cog
  time   : "jm_time"                                             '   timing and delays
  dtmr   : "jm_time"                                             '   display state timer
  scrtmr : "jm_time"                                             '   scroll timer
  shktmr : "jm_time"                                             '   shake hold-off timer
  io     : "jm_io"                                               '   io control
  ee     : "jm_24xx512"                                          '   64k eeprom access
  acc    : "jm_mma7660fc"                                        '   MMA7660 accelerometer
  pads   : "jm_touchpads"                                        '   touchpad input
  oled   : "oled_asmfast"                                        ' * OLED driver (TEMPORARY)
  leds   : "jm_ebadge_leds"                                      ' * charlieplexing driver for LEDs
  ir     : "tt_ir_hdserial"                                      ' * half-duplex serial via IR
  term   : "jm_fullduplexserial"                                 ' * full-duplex serial via terminal

                                                                  
' * Uses cog when loaded                                          


var

  long  cpcog                                                    ' charlieplex cog (>0 if loaded)

  long  badgemode                                                ' 0 for name/ir, 1 for accelerometer

  long  dstate                                                   ' display state
  long  msgms                                                    ' message/display milliseconds

  long  numcontacts                                              ' number of contacts in EEPROM

  long  scrollidx                                                ' scroll position
  long  scrollidxmax
  byte  scroll[32]                                               ' scroll buffer

  byte  buf1[CONTACT_LEN]                                        ' rx buffer
  byte  buf2[CONTACT_LEN]                                        ' check buffer
  byte crc


con

  SCROLL_NAME = NO                                               ' scroll status for message mode 0

pub main | t
'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed
  dtmr.start
  scrtmr.start
  shktmr.start

  io.start($0000, $0000)                                         ' clear all pins (this cog)
  
  ee.start(%000)                                                 ' connect to boot eeprom

  acc.start(SCL, SDA)                                            ' connect to accelerometer

  pads.start(TPCount, @TPPins, TPDischarge)                      ' setup touchpad interface

  oled.init(OLED_CS, OLED_DC, OLED_DAT, OLED_CLK, OLED_RST, {    ' start buffered oled driver
}           oled#SSD1306_SWITCHCAPVCC, oled#TYPE_128X64)
  oled.AutoUpdateOff  

  cpcog := leds.start                                            ' start charlieplexing driver


  ir.start(IR_IN, IR_OUT, IR_BAUD, IR_FREQ)                      ' start IR serial 
  
  term.start(RX1, TX1, %0000, T_BAUD)                            ' start serial for terminal

'  set_1_rgb(1)
'  set_1_rgb(2)

  dstate := MSG_LOGO                                             ' show OSH logo 1st
  update_display(1000)

 
repeat 5
  sendcolor(@irreset)
  waitcnt(clkfreq / 15 + cnt)

'sendcolor(@irtest)

repeat
  sendcolor(@irreset)
  waitcnt(clkfreq / 25 + cnt)
   
  sendcolor(@irred)
  leds.set_rgbx(leds#RED, leds#RED)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@ircyan)
  leds.set_rgbx(leds#CYAN, leds#CYAN)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@irpurple)
  leds.set_rgbx(leds#MAJENTA, leds#MAJENTA)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@irgreen)
  leds.set_rgbx(leds#GREEN, leds#GREEN)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@irblue)
  leds.set_rgbx(leds#BLUE, leds#BLUE)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@irwhite)
  leds.set_rgbx(leds#WHITE, leds#WHITE)
  waitcnt(clkfreq / 1 + cnt)

  sendcolor(@iryellow)
  leds.set_rgbx(leds#YELLOW, leds#YELLOW)
  waitcnt(clkfreq / 1 + cnt)
   
 
{
 repeat
   term.hex(ir.rx,2)
 }
pub update_display(ms) | p_str0, p_str1

'' Displays "screen" based on dstate
'' -- if no match, Propeller beanie logo displayed
'' -- ms is milliseconds to display message   

{  case dstate
    MSG_NAME:
      if (SCROLL_NAME == NO)                                     ' non-scrolling name
        p_str0 := @Name_0
        p_str1 := @Name_1 
      else
        bytefill(@scroll, " ", 8)                                ' blanks
        p_str0 := @scroll
        p_str1 := @scroll
        
    MSG_TX_NOW:
      p_str0 := @Msg1_0                                          ' sending contact
      p_str1 := @Msg1_1

    MSG_TX_DONE:
      p_str0 := @Msg2_0                                          ' contact sent
      p_str1 := @Msg2_1
      
    MSG_RX_NOW:
      p_str0 := @Msg3_0                                          ' receiving contact
      p_str1 := @Msg3_1

    MSG_RX_DONE:
      p_str0 := @Msg4_0                                          ' contact received
      p_str1 := @Msg4_1
      
    MSG_RX_DUP:
      p_str0 := @Msg5_0                                          ' already stored
      p_str1 := @Msg5_1

    MSG_RX_ERROR:
      p_str0 := @Msg6_0                                          ' receive error
      p_str1 := @Msg6_1
      
    MSG_WIPE:
      p_str0 := @Msg9_0                                          ' database wipe
      p_str1 := @Msg9_1

    other: { graphic }  }
      bytemove(oled.getBuffer, @Logo, 1024)
    
  if (dstate => 0)
    'oled.clearDisplay
    bytefill(oled.getBuffer, 0, 1024)
    oled.write2x8String(p_str0, strsize(p_str0) <# 8, 0)  
    oled.write2x8String(p_str1, strsize(p_str1) <# 8, 1)

  oled.updateDisplay

  msgms := ms                                                    ' set message time
  dtmr.start                                                     ' restart display timer

pub set_1_blue(idx)                                              ' LOCKED 30 JUN 2015  

'' Set one of the six blue LEDs
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?
    if ((idx => BLUE_0) and (idx =< BLUE_5))                     ' valid element?
      leds.set_blue(1 << idx)                                    ' set via mask
    else
      leds.set_blue(%000000)                                     ' clear all 
    return 

  ' if no driver loaded, use direct control

  case idx
    BLUE_0:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_1:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L    
      dira[BLU_CP2..BLU_CP0] := %101

    BLUE_2:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H 
      dira[BLU_CP2..BLU_CP0] := %101      

    BLUE_3:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z  
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_4:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[BLU_CP2..BLU_CP0] := %011
      
    BLUE_5:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H
      dira[BLU_CP2..BLU_CP0] := %011  

    other:
      outa[BLU_CP2..BLU_CP0] := %000                             ' disable all
      dira[BLU_CP2..BLU_CP0] := %000         


pub set_1_rgb(idx)                                               ' LOCKED 30 JUN 2015  

'' Set one of the LEDs within the RGB modules
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs/colors use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?      
    if ((idx => RGB_B1) and (idx =< RGB_R2))                     ' valid element?                  
      leds.set_rgb(1 << idx)                                     ' set via mask                    
    else                                                                                           
      leds.set_rgb(%000000)                                      ' clear all                       
    return

  ' if no driver loaded, use direct control

  case idx
    RGB_B1:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %011
  
    RGB_R1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[RGB_CP2..RGB_CP0] := %110

    RGB_B2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H  
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H 
      dira[RGB_CP2..RGB_CP0] := %011      

    RGB_R2:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z
      dira[RGB_CP2..RGB_CP0] := %110  
  
    other:
      outa[RGB_CP2..RGB_CP0] := %000                             ' disable all
      dira[RGB_CP2..RGB_CP0] := %000 


pub crc8_maxim(array,numchar) | crcvalue, addr, t1, i
  crcvalue := 0
  i := 0
    repeat numchar
      addr :=  byte[array][i]
       i++
      repeat 8
        t1 := addr
        addr ->= 1
        t1 ^= crcvalue
        crcvalue ->= 1
        if (t1 & 1) == 1
          crcvalue ^= $8C
      
  return crcvalue   
pub sendcolor(ird) | t
  t := 0
  
  repeat byte[ird][0]
   t++
   ir.tx(byte[ird][t])
  ir.tx(crc8_maxim(ird+1,byte[ird][0]))


  
DAT
irreset byte 02, $90, $24
irblue byte 02, $90, $61 
irpurple byte 04, $91, $0E, $08, $9D
irgreen byte 02, $90, $62
irred byte 02, $90, $64
irtest byte 32, $96, $19, $0E, $3F, $0A, $29, $82, $04, $44, $9C, $26, $0C, $2A, $67, $6F, $58, $EE, $48, $03, $D1, $3C, $07, $05, $DF, $96, $19, $0E, $3F, $0A, $29, $82, $04
ircyan byte 02,$90, $63
iryellow byte 02, $90, $66
irwhite byte 02, $90, $67 

{irdata byte $90, $62, $1A  
irdata byte $96, $19, $10, $24, $0E 
irdata byte $96, $19, $10, $24, $0E, $F5, $F6, $03, $A2, $99, $26, $64, $6C, $D1, $42, $07, $20, $D0, $32, $F0, $11, $96, $19, $10, $24, $0E, $F5, $F6, $03, $A2
 }

DAT { OSH Logo }
  Logo
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $C4, $E0, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0, $40
byte $60, $20, $20, $30, $10, $10, $18, $08, $08, $08, $08, $8C, $8C, $04, $04, $04
byte $04, $00, $EC, $FF, $FF, $FF, $FF, $FE, $00, $08, $08, $18, $10, $10, $20, $20
byte $60, $40, $C0, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $80, $40, $20, $10, $08, $04, $02, $03, $01, $00, $00, $00
byte $00, $90, $00, $00, $00, $00, $00, $00, $00, $00, $00, $F0, $FC, $F8, $C0, $E0
byte $F0, $F0, $FB, $FB, $FB, $FB, $FB, $FB, $70, $00, $00, $10, $00, $00, $00, $00
byte $00, $80, $00, $00, $01, $01, $02, $04, $08, $10, $20, $40, $80, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80
byte $E0, $38, $04, $03, $00, $00, $00, $00, $10, $10, $00, $00, $00, $00, $00, $68
byte $6E, $6F, $6F, $6C, $60, $00, $40, $60, $6E, $60, $60, $6E, $6F, $6F, $6F, $6F
byte $6F, $6F, $6F, $6F, $6F, $6F, $6F, $6F, $6E, $68, $60, $6F, $6F, $6E, $00, $00
byte $60, $6C, $68, $40, $00, $00, $00, $00, $00, $00, $00, $00, $00, $03, $0C, $38
byte $E0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $C0, $80, $3C, $07
byte $00, $00, $00, $00, $00, $40, $60, $70, $76, $77, $76, $70, $70, $70, $76, $76
byte $76, $36, $B6, $B6, $B6, $B6, $B6, $B6, $B6, $B6, $B6, $B6, $B6, $76, $76, $76
byte $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76, $76
byte $76, $76, $76, $76, $76, $76, $76, $74, $60, $40, $00, $00, $00, $00, $00, $00
byte $00, $07, $7C, $C0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $C0
byte $F0, $F8, $1C, $06, $03, $C1, $E0, $20, $E0, $E0, $C0, $00, $78, $71, $07, $FE
byte $F8, $00, $00, $00, $80, $80, $36, $76, $76, $76, $76, $76, $76, $76, $76, $74
byte $71, $73, $74, $76, $76, $76, $76, $76, $75, $75, $E5, $F3, $13, $73, $77, $76
byte $6E, $4C, $1C, $3A, $72, $F6, $E6, $C6, $92, $7C, $66, $6A, $6E, $65, $64, $36
byte $1E, $4C, $60, $76, $76, $76, $76, $76, $76, $62, $00, $80, $80, $00, $00, $00
byte $00, $00, $00, $3F, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $7F
byte $FF, $C0, $C0, $80, $7F, $FF, $E0, $80, $9F, $8F, $83, $C0, $E0, $F0, $FC, $7F
byte $9F, $F0, $F4, $33, $31, $7F, $FE, $00, $FE, $FF, $E0, $E3, $03, $F6, $F6, $F7
byte $07, $07, $07, $00, $1C, $34, $66, $46, $C3, $00, $FF, $FF, $00, $03, $07, $07
byte $06, $16, $1E, $8C, $80, $C0, $E3, $FF, $7F, $1C, $F8, $FF, $00, $04, $6E, $FE
byte $9B, $BB, $BB, $DB, $DB, $F3, $60, $FE, $FF, $1E, $7C, $F1, $FF, $FF, $18, $F2
byte $FF, $DF, $DF, $DB, $DB, $03, $B0, $FC, $FE, $FB, $3C, $9E, $E6, $76, $3E, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $01, $01, $03, $03, $03, $03, $03, $01, $01, $00, $00
byte $01, $01, $00, $00, $00, $00, $01, $00, $00, $00, $00, $00, $00, $03, $07, $07
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $07, $07, $00, $03, $03, $03
byte $03, $03, $03, $03, $01, $01, $00, $00, $00, $00, $00, $01, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01
byte $01, $01, $01, $00, $00, $7C, $FF, $7F, $07, $00, $00, $00, $00, $00, $00, $00
byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00